// -----------------------------------------------------------
// ReceivingMessageListener.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.bscomm;


import cz.vutbr.wsagent.comm.Comm;
import cz.vutbr.wsagent.comm.IReceivingListener;
import cz.vutbr.wsagent.comm.TransPacket;
import cz.vutbr.wsagent.db.hibernate.DAOFactory;
import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import cz.vutbr.wsagent.entity.Message;
import cz.vutbr.wsagent.entity.Mote;
import java.awt.event.ActionEvent;
import java.util.Date;

/**
 * Listener for receiving messages.
 * @author Martin Gábor
 */
public class ReceivingMessageListener implements IReceivingListener{

    private Comm comm = null;

    /**
     * Constructor. It contains an instance of Comms class to get received messages.
     * Registers listener in Comm class to listen to incoming messages.
     * @param comm Instance of class that receives messages from Basestation.
     */
    public ReceivingMessageListener(Comm comm){
        this.comm = comm;
        this.comm.addReceivingListener(this);
    }


    /**
     * This method inserts incoming message to hibernate database. Mote that sent it
     * should already exist in the database, otherwise is created too.
     * @param event Event
     */
    public void actionPerformed(ActionEvent event) {

        String address = event.getActionCommand();
        
        Message message = new Message();
        Mote mote = null;

        TransPacket messageValue = null;
        try{
            messageValue = comm.getMessage(address);
        } catch(Exception e){
        }
        try {
            // find out if there exists a record in DB.
            mote = DAOFactory.getDAOFactory().getMoteDAO().read(address);
        } catch (DatabaseErrorException ex) {
        }
        
        // if record doesnt exist
        if(mote == null){
            mote = new Mote();
            mote.setAddress(address);
            // save new mote
            DAOFactory.getDAOFactory().getMoteDAO().save(mote);
        }

        String content = messageValue.getContent();

        // if it is agent, get the last character
        int length = content.length();
        if(messageValue.isIsAgent())
            content = content.substring(0,length-1);

        // if it is empty do not save it.
        if(content.isEmpty())
            return;

        // date
        message.setDate(new Date());
        message.setValue(content);
        message.setMote(mote);
        message.setIsNew(true);
        message.setDestAddress(messageValue.getSecondAddress());

        // save message to DB
        DAOFactory.getDAOFactory().getMessageDAO().save(message);

        System.out.println("Received message successfully inserted into database.");
    }

}
