// -----------------------------------------------------------
// AgentDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.db.hibernate;

import cz.vutbr.wsagent.entity.Agent;
import cz.vutbr.wsagent.entity.IAgentDAO;
import java.util.List;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * This class implements IAgentDAO interface. It supports data access to agent object of
 * hibernate framework.
 *
 * @author Martin Gábor
 */
public class AgentDAO implements IAgentDAO {


    // session object
    Session session = null;


    /**
     * Constructor for creating hibernate session using <code>DbConnFactory</code>.
     */
    public AgentDAO() {
        this.session = DbConnFactory.getSession();
    }


    /**
     * This method is used for storing new agent to hibernate database.
     *
     * @param agent Agent.
     * @see Agent
     */
    public void save(Agent agent) {
        session.beginTransaction();
        session.save(agent);
        session.getTransaction().commit();
    }

    /**
     * This method updates existing agent in hibernate database.
     *
     * @param agent Agent to be updated.
     * @see Agent
     */
    public void update(Agent agent) {
        session.beginTransaction();
        session.merge(agent);
        session.getTransaction().commit();
    }

    /**
     * This method is used to delete agent from hibernate database.
     *
     * @param agent Agent.
     * @see Agent
     */
    public void delete(Agent agent) {
        session.beginTransaction();
        session.delete(agent);
        session.flush();
    }


    /**
     * This method is used to get all agents from hibernate database.
     *
     * @return Returns list of all agents.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Agent
     */
    public List<Agent> readAll() throws DatabaseErrorException{
        List<Agent> agentList = null;
        try {
            session.beginTransaction();
            Query q = session.createQuery("from Agent");
            agentList = (List<Agent>) q.list();
        } catch (Exception e) {
            throw new DatabaseErrorException();
        }
        return agentList;
    }

    /**
     * This method is used to get concrete agent from hibernate database.
     *
     * @param agentId The id of agent that should be returned.
     * @return Returns required agent.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Agent
     */
    public Agent read(long agentId) throws DatabaseErrorException {

        Agent agent = null;

        try {
            session.beginTransaction();
            Query q = session.createQuery("from Agent where id = :agentId");
            q.setLong("agentId", agentId);
            agent = (Agent) q.uniqueResult();
        } catch (Exception e) {
            throw new DatabaseErrorException();
        }

        if(agent == null)
            throw new DatabaseErrorException();

        return agent;
    }

}
