// -----------------------------------------------------------
// MoteDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.db.hibernate;

import cz.vutbr.wsagent.entity.IMoteDAO;
import cz.vutbr.wsagent.entity.Mote;
import java.util.List;
import org.hibernate.Query;
import org.hibernate.Session;
/**
 * This class implements IMoteDAO interface. It supports data access to Mote object of
 * hibernate framework.
 * @author Martin Gábor
 */
public class MoteDAO implements IMoteDAO{

    Session session = null;

    /**
     * Constructor for creating hibernate session using <code>DbConnFactory</code>.
     */
    public MoteDAO() {
        this.session = DbConnFactory.getSession();
    }

    /**
     * This method is used for storing new mote to hibernate database.
     *
     * @param mote Mote.
     * @see Mote
     */
    public void save(Mote mote) {
        session.beginTransaction();
        session.save(mote);
        session.getTransaction().commit();
    }

    /**
     * This method updates existing mote in hibernate database.
     *
     * @param mote Mote to be updated.
     * @see Mote
     */
    public void update(Mote mote) {
        session.beginTransaction();
        session.merge(mote);
        session.getTransaction().commit();
    }


    /**
     * This method is used to delete mote from hibernate database.
     *
     * @param mote Mote.
     * @see Mote
     */
    public void delete(Mote mote) {
        session.beginTransaction();
        session.delete(mote);
        session.flush();
    }

    /**
     * This method is used to get all motes from hibernate database.
     *
     * @return Returns list of all motes.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Mote
     */
    public List<Mote> readAll() throws DatabaseErrorException{
        List<Mote> moteList = null;
        try {
            session.beginTransaction();
            Query q = session.createQuery("from Mote order by address");
            moteList = (List<Mote>) q.list();
        } catch (Exception e) {
            throw new DatabaseErrorException();
        }
        return moteList;
    }

    /**
     * This method is used to get mote from hibernate database.
     *
     * @param address The address of required mote.
     * @return Returns required Message object.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     */
    public Mote read(String address) throws DatabaseErrorException{

        Mote mote = null;

        try {
            session.beginTransaction();
            Query q = session.createQuery("from Mote where address = ?");
            q.setString(0, address);
            mote = (Mote) q.uniqueResult();
        } catch (Exception e) {
            throw new DatabaseErrorException();
        }


        return mote;
    }

}
