// -----------------------------------------------------------
// IMoteDAO.java
// -----------------------------------------------------------

package cz.vutbr.wsagent.entity;

import cz.vutbr.wsagent.db.hibernate.DatabaseErrorException;
import java.util.List;

/**
 * The class representing a mote DAO.
 * @author Martin Gábor
 */
public interface IMoteDAO {

    /**
     * This method is used for storing new mote to database.
     *
     * @param mote Mote.
     * @see Mote
     */
    void save(Mote mote);

    /**
     * This method updates existing mote in database.
     *
     * @param mote Mote to be updated.
     * @see Mote
     */
    void update(Mote mote);

    /**
     * This method is used to delete mote from database.
     *
     * @param mote Mote.
     * @see Mote
     */
    void delete(Mote mote);

    /**
     * This method is used to get all motes from database.
     *
     * @return Returns list of all motes.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Mote
     */
    List<Mote> readAll() throws DatabaseErrorException;

    /**
     * This method is used to get mote from database.
     *
     * @param address The address of required mote.
     * @return Returns required Message object.
     * @throws DatabaseErrorException Thrown when an error occurs while database processing.
     * @see Message
     */
    Mote read(String address) throws DatabaseErrorException;
}
